/* StribogSum */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "Stribog.h"

int main(int argc, char *argv[])
{
	unsigned char *buf;;
	char digest[64];
	long size;
	size_t len;
	unsigned char c;
	int i;
	STRIBOG_CTX ctx;
	
	FILE *file;

	if (argc != 2)
	{
		printf("Usage: %s <input file>\n", argv[0]);
		return -1;
	}

	/* Open the file */
	file = fopen(argv[1], "rb");
	if (!file)
	{
		fprintf(stderr, "Error opening the file\n");
		return -1;
	}
	
	/* Get file size */
	fseek(file, 0, SEEK_END);
	size = ftell(file);
	fseek(file, 0, SEEK_SET);

	/* Allocate entire file buffer at once */
	buf = (unsigned char *) calloc(size, sizeof(unsigned char));
	if (!buf)
	{
		fprintf(stderr, "Error allocating buffer\n");
		return -1;
	}

	/* Read the file and add compute its Stribog hash */
	STRIBOG_init(&ctx);
	for (i = 0; i < size; i++)
	{
		len = fread(&c, sizeof(unsigned char), 1, file);
		if (len <= 0)
		{
			fprintf(stderr, "Error reading the file\n");
			return -1;
		}
		buf[i] = c;
	}
	STRIBOG_add(&ctx, (unsigned char *)buf, size);
	STRIBOG_finalize(&ctx, (unsigned char *)digest);
	
	fclose(file);

	/* Print the hash */
	for (i = 0; i < 64; i++)
	{
		printf("%02x", (unsigned char)digest[i]);
	}
	printf("\n");

	return 0;
}